import { ContainerConstants } from '@medjaibot/framework/constants/ContainerConstants';
import { Logger } from '@medjaibot/framework/logger/Logger';
import { IInitializationContext } from '@medjaibot/framework/plugin/IInitializationContext';
import { InitializationSide } from '@medjaibot/framework/plugin/InitializationSide';
import { Plugin } from '@medjaibot/framework/plugin/Plugin';
import { inject, injectable } from 'inversify';
import 'reflect-metadata';

/**
 * A basic test plugin
 *
 * @export
 * @class TestPlugin
 * @extends {Plugin}
 */
@injectable()
export default class TestPlugin extends Plugin {
    private logger: Logger;

    constructor(
        @inject(ContainerConstants.LOGGING.LOGGER)
        logger: Logger,
    ) {
        super(
            'com.github.MedjaiBot.plugins.TestPlugin',
            'TestPlugin',
            '0.0.1',
            'Yannick Fricke and contributors',
        );

        this.logger = logger;
    }

    /**
     * Initializes the test plugin
     *
     * @param {IInitializationContext} initializationContext The initialization context
     * @memberof TestPlugin
     */
    public onInit(
        initializationContext: IInitializationContext,
    ) {
        if (initializationContext.initializationSide === InitializationSide.SERVER) {
            this.logger.info(
                `${TestPlugin.name}: Initializing from the server side`,
            );
        } else {
            this.logger.info(
                `${TestPlugin.name}: Initializing from the client side`,
            );
        }
    }
}
